﻿using AD_MAUI.Model;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Runtime.CompilerServices;

namespace AD_MAUI.ViewModel
{
    // TODO make sure not to notify change if not necessary!

    public class AlbumViewModel : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler? PropertyChanged;

        private readonly Album album = new();

        public int Id { get => album.Id; }

        public string Title
        {
            get => album.Title;
            set
            {
                if (album.Title != value)
                {
                    album.Title = value;
                    OnPropertyChanged();
                }
            }
        }

        public string Artist
        {
            get => album.Artist;
            set
            {
                if (album.Artist != value)
                {
                    album.Artist = value;
                    OnPropertyChanged();
                }
            }
        }

        public string CoverImage
        {
            get => album.CoverImage;
            set
            {
                if (album.CoverImage != value)
                {
                    album.CoverImage = value;
                    OnPropertyChanged();
                }
            }
        }

        public ReadOnlyObservableCollection<SongViewModel> Songs { get => new(songs); }

        private readonly ObservableCollection<SongViewModel> songs;


        public AlbumViewModel(Album? album)
        {
            // Mocked data for testing
            this.album = album ?? new Album
            {
                Id = 1,
                Title = "Test Album",
                Artist = "Test Artist",
                CoverImage = "dotnet_bot.png", // make this a path?
                Songs = new List<Song>
                {
                    new Song { Id = 1, Title = "Test Song 1", Duration = 210 },
                    new Song { Id = 2, Title = "Test Song 2", Duration = 260 },
                    new Song { Id = 3, Title = "Test Song 3", Duration = 817 },
                }
            };
            songs = new ObservableCollection<SongViewModel>(this.album.Songs.Select(song => new SongViewModel(song)));
        }

        protected virtual void OnPropertyChanged([CallerMemberName] string? propertyName = null)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}